@@ -15,14 +15,57 @@
 #
 
 package MongoDB::Async::OID;
-our $VERSION = '0.45';
-# ABSTRACT: A Mongo Object ID
+{
+  $MongoDB::Async::OID::VERSION = '0.702.2';
+}
 
-use Mouse;
+# ABSTRACT: A Mongo Object ID
+use bytes; # length in bytes
+use Carp qw/croak/;
+
+sub new {
+	my $self = shift;
+	$self = bless( ( (ref($_[0]) eq 'HASH')	?  $_[0]  :  { (int(@_) == 1) ? ( 'value' => @_ ) : @_ }	), $self);
+	
+	unless ( $self->{value} ){
+		$self->{value} = _generate_oid; 
+	}elsif(length($self->{value}) != 24){
+		croak("OIDs need to have a length of 24 bytes")
+	}
+	
+	$self;
+}
+
+sub value {$_[0]->{value}}
+*to_string = \&value;
+
+
+
+sub get_time {
+    return hex(substr($_[0]->{value}, 0, 8));
+}
+
+
+sub TO_JSON { {'$oid' => $_[0]->{value}} }
+
+use overload
+    '""' => \&to_string,
+    'fallback' => 1;
+
+
+1;
+
+__END__
+
+=pod
 
 =head1 NAME
 
-MongoDB::Async::OID - A Mongo ObjectId
+MongoDB::Async::OID - A Mongo Object ID
+
+=head1 VERSION
+
+version 0.702.2
 
 =head1 SYNOPSIS
 
@@ -42,10 +85,15 @@ constructor.  For example:
 
     my $id1 = MongoDB::Async::OID->new;
     my $id2 = MongoDB::Async::OID->new(value => $id1->value);
+	my $id3 = MongoDB::Async::OID->new({value => $id1->value});
 
 Now C<$id1> and C<$id2> will have the same value.
 
-Warning: at the moment, OID generation is not thread safe.
+OID generation is thread safe.
+
+=head1 NAME
+
+MongoDB::Async::OID - A Mongo ObjectId
 
 =head1 SEE ALSO
 
@@ -60,28 +108,6 @@ It is a 24-character hexidecimal string (12 bytes).
 
 Its string representation is the 24-character string.
 
-=cut
-
-has value => (
-    is      => 'ro',
-    isa     => 'Str',
-    required => 1,
-    builder => 'build_value',
-);
-
-sub BUILDARGS { 
-    my $class = shift; 
-    return $class->SUPER::BUILDARGS(flibble => @_)
-        if @_ % 2; 
-    return $class->SUPER::BUILDARGS(@_); 
-}
-
-sub build_value {
-    my $self = shift;
-
-    _build_value($self, @_ ? @_ : ());
-}
-
 =head1 METHODS
 
 =head2 to_string
@@ -90,13 +116,6 @@ sub build_value {
 
 Gets the value of this OID as a 24-digit hexidecimal string.
 
-=cut
-
-sub to_string {
-    my ($self) = @_;
-    $self->value;
-}
-
 =head2 get_time
 
     my $date = DateTime->from_epoch(epoch => $id->get_time);
@@ -104,18 +123,6 @@ sub to_string {
 Each OID contains a 4 bytes timestamp from when it was created.  This method
 extracts the timestamp.  
 
-=cut
-
-sub get_time {
-    my ($self) = @_;
-
-    my $ts = 0;
-    for (my $i = 0; $i<4; $i++) {
-        $ts = ($ts * 256) + hex(substr($self->value, $i*2, 2));
-    }
-    return $ts;
-}
-
 =head2 TO_JSON
 
     my $json = JSON->new;
@@ -129,22 +136,34 @@ representation used by MongoDB, that is, an OID with the value
 "012345678901234567890123" will be represented as 
 C<{"$oid" : "012345678901234567890123"}>.
 
-=cut
-
-sub TO_JSON {
-    my ($self) = @_;
-    return {'$oid' => $self->value};
-}
-
-use overload
-    '""' => \&to_string,
-    'fallback' => 1;
-
-no Any::Moose;
-__PACKAGE__->meta->make_immutable;
-
-1;
-
 =head1 AUTHOR
 
   Kristina Chodorow <kristina@mongodb.org>
+
+=head1 AUTHORS
+
+=over 4
+
+=item *
+
+Florian Ragwitz <rafl@debian.org>
+
+=item *
+
+Kristina Chodorow <kristina@mongodb.org>
+
+=item *
+
+Mike Friedman <mike.friedman@10gen.com>
+
+=back
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2012 by 10gen, Inc..
+
+This is free software, licensed under:
+
+  The Apache License, Version 2.0, January 2004
+
+=cut
